package com.ruoyi.framework.recovery.service;

import com.ruoyi.framework.recovery.domain.RecContent;
import com.ruoyi.framework.recovery.domain.vo.WxContent;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 评论Service接口
 *
 * @author jin
 * @date 2024-11-16
 */
public interface IRecContentService
{
    /**
     * 查询评论
     *
     * @param id 评论主键
     * @return 评论
     */
    public RecContent selectRecContentById(Long id);

    /**
     * 查询评论列表
     *
     * @param recContent 评论
     * @return 评论集合
     */
    public List<RecContent> selectRecContentList(RecContent recContent);

    /**
     * 新增评论
     *
     * @param recContent 评论
     * @return 结果
     */
    public int insertRecContent(RecContent recContent);

    /**
     * 修改评论
     *
     * @param recContent 评论
     * @return 结果
     */
    public int updateRecContent(RecContent recContent);

    /**
     * 批量删除评论
     *
     * @param ids 需要删除的评论主键集合
     * @return 结果
     */
    public int deleteRecContentByIds(Long[] ids);

    /**
     * 删除评论信息
     *
     * @param id 评论主键
     * @return 结果
     */
    public int deleteRecContentById(Long id);

    /**
     * 记录用户点赞
     *
     * @param wxOpenid 微信唯一标识
     * @param id 评论id
     * @return 结果集
     */
    int insertRecContentUser(String wxOpenid, Long id);

    /**
     * 查询小程序评论
     *
     * @param wxContentVo 入参
     * @return 结果集
     */
    List<WxContent> selectWxContentList(WxContent wxContentVo);

    /**
     * 查询当前用户点过赞
     *
     * @param wxOpenid 用户Id
     * @return 结果集
     */
    List<WxContent> selectRecContentByOpenId(String wxOpenid);
}
